/*
 * ANELEX Interface Microcontroller
 * Test program level
 * Started: 28. August 2010, im Museum
 * Codebase: Gamma10 test program level
 *
 */

#include "toolchain.h"
#include "anelex_code.h"

void Initialize()
{
	// negierte Ausgaenge schalten
	PORTA = 0x00;
	PORTB = 0x00; // LED (auslassen, an=0x01)
	PORTC = 0x00;
	PORTD = 0x00;

	// DDRA: DATA-Anschluss von 0..5
	DDRA = (1 << DDA0) | (1 << DDA1) | (1 << DDA2) | (1 << DDA3) | (1 << DDA4) | (1 << DDA5);
	// DDRB: LED als output, CA ist input, IB2 ist input
	DDRB = (1 << DDB0);
	// DDRC: STROBE, CP, PRINTCM, RESET sind output, ERRORS input
	DDRC = (1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3);
	// DDRD: AI und KI sind Output, IB1, IB2 input
	DDRD = (1 << DDD6) | (1 << DDD5);

	// jetzt erst default values setzen
	PORTC = (1 << PORTC0) | (1 << PORTC1) | (1 << PORTC2) | (1 << PORTC3);

	// Pin Change Intterupts aufsetzen
	// PCMSK*: Pin Change Masks (welche Pins tragen zum Intterupt bei)
	// PORT C:
	  //PCMSK2 = (1 << PCINT16) | (1 << PCINT17) | (1 << PCINT18) | (1 << PCINT19);
	// Pin Change Interrupt Control Register: PC Interrupt Enable for Port C
	  //PCICR = (1 << PCIE2);

	// INTTERUPTS AUFSETZEN
	//EICRA = (1 << ISC20) | (1 << ISC21) // rising FLANKE von INT2 (CA = Load Cycle) erzeugt intterrupt
	//		| (1 << ISC11) | (1 << ISC10); // rising edge von INT1 (LINE READY) erzeugt intteruupt

	//EIMSK = (1 << INT2) | (1 << INT1); // interrupt fuer INT1,2 anschalten


	// Interrupt enable
	sei();
}

#define LINE_FINISHED -2  // alle zeichen reingehauen
#define LINE_PRINTED -3  // Zeile gedruckt
volatile int current_counter; // Numerischer Index: von 0 bis 120 zeichen, wo wir gerade sind.
char* current_string;

//ISR(INT2_vect) {
void print_zeichen() {
	// Communication Announcement steigende Flanke.
	// je nach zustand
	if(current_counter >= 0 && current_counter <= 144) {
		// aussage: Naechsetes Zeichen kann angelegt werden
		char orig = current_string[ current_counter ];
		if(orig) {
			// normales zeichen
			char transformed = ~ anelex_code[ orig ];
			printf("%d: Zeichen %c, entspricht oktal %o\n", current_counter, orig, transformed);
			//printf("%d", current_counter, orig);
			//_delay_us(100);
			
			// LEGE ZEICHEN AN:
			PORTA = transformed;
			_delay_us(10); 
			// STROBE anmachen (auf 0 legen)
			PORTC &= ~(1 << PORTC0);
			_delay_us(1);
			PORTC |= (1 << PORTC0); // ausmachen
			// jetzt bis zum naechsten Zeichen warten

			current_counter++;
			return;
		} else {
			// ZEILENENDE ERREICHT
			current_counter = LINE_FINISHED;
			PORTA = ~ 0x01;// aufraeumen am DATENPORT
			printf("Print Command\n");

			// PRINT COMMAND fuer Zeile machen
			PORTC &= ~(1 << PORTC2);
			_delay_us(10);
			PORTC |= (1 << PORTC2);
			// warten auf auf das naechste CA.
		}
	} else if(current_counter == LINE_FINISHED) {
		//_delay_ms(2);
		//if((PINB & (1 << PINB2)) != 0) {
			// zeichen ist noch an, d.h. zeile fertig
		
		//}
	} else {
		//printf("Fehler - hier stimmt was nicht");
	}
}


//ISR(INT1_vect) {
void print_zeile() {
	// Rising Edge von Line Ready = Zeile wird gedruckt!
	printf("Zeile wird gedruckt...\n");
	//while((PIND & (1 << PIND3)) != 0);
	_delay_ms(70);
	printf("Zeile fertig gedruckt\n");
	current_counter = LINE_PRINTED;		
}

char* tests[] = {

		"HALLOWELT",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		"0123456789",
		"ZYWVUTSRQPONMLKJIHGFEDCBA",
		"9876543210",
		"+-.,()/=?",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ",
		"0123456789012345678901234567890123456789012345678901234567890123456789",
		"0123456789012345678901234567890123456789012345678901234567890123456789",
		"ABCDEFABCDEFABCDEFABCDEFABCDEFABCDEFABCDEFABCDEFABCDEFABCDEFABCDEF",
		"0123456789012345678901234567890123456789012345678901234567890123456789",
		"a",
		"01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789",
		"b",
		"01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789",
		"01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789",
		"                                     MITTIG    AUSGERICHTET                            TOLL",
		"0123456789 023456 0123456789 0123456789 0123 456 789",
		"HALLO DIES IST EIN TEST",
		
		"1234567890123456789012345678901234567890123456789012345678901234567890",
		"123456789012345678901234567890123456789012345678901234567890",
		"12345678901234567890123456789012345678901234567890",
		"1234567890123456789012345678901234567890",
		"123456789012345678901234567890",
		"12345678901234567890",
		"1234567890",
		"123",
		
		""
	};

	
void print_line_feed() {
	// Lege Zeilenvorschub an
	PORTA = ~ 0x01;
	_delay_us(800);
	// STROBE anmachen (auf 0 legen)
	PORTC &= ~(1 << PORTC0);
	_delay_us(1);
	PORTC |= (1 << PORTC0); // ausmachen
}

int main() {
	int counter = 0; // globaler Counter

	Initialize();
	UartInit(38400);
	setup_stdio();

	stop_led();


	byte_t y;
	for(y=0;;y++) {
		// warten, bis BUTTON gedrueckt ist
		if(PINB & (1 << PORTB4)) {
			start_led();
			for(;;) {
				if((PINB & (1 << PORTB4)) == 0)
					break;
			}
			printf("%d: Button gedrueckt\n", counter++);

			// hier koennen wir jetzt mal arbeiten
			// alle strings durchdrucken
			for(int s=0; tests[s][0]; s++) {
				current_string = tests[s];
				printf("%d. Drucke %s\n", s, current_string);

				print_line_feed();
				current_counter = 0;

				for(;;) {
					print_zeichen();

					_delay_us(10);

					if(current_counter == LINE_FINISHED) {
						printf("Zeile fertig ruebergeschoben. starte drucken.\n");
						break;
					}
				}

				_delay_ms(500);
				//print_zeile();

				stop_led();

			} // for tests

		} // for buton
		//printf("%d: Sleeping...\n", counter);
		//counter++;
		//_delay_ms(10);
		// endlosschleife
	}

	return 0;
}
