/*
 * ANELEX Interface Microcontroller
 * Test program level
 * Started: 28. August 2010, im Museum
 * Codebase: Gamma10 test program level
 *
 */

#include "wiring.h"
#include "toolchain.h"
#include "frontend.h"

static FILE output = FDEV_SETUP_STREAM(uart_transmit, NULL, _FDEV_SETUP_WRITE);

void Initialize()
{
	// negierte Ausgaenge schalten
	PORTA = 0x00;
	PORTB = 0x00; // LED (auslassen, an=0x01)
	PORTC = 0x00;
	PORTD = 0x00;

	// DDRA: DATA-Anschluss von 0..5
	DDRA = (1 << DDA0) | (1 << DDA1) | (1 << DDA2) | (1 << DDA3) | (1 << DDA4) | (1 << DDA5);
	// DDRB: LED als output, CA ist input, IB2 ist input
	DDRB = (1 << DDB0);
	// DDRC: STROBE, CP, PRINTCM, RESET sind output, ERRORS input
	DDRC = (1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3);
	// DDRD: AI und KI sind Output, IB1, IB2 input, RTS input, CTS output
	DDRD = (1 << DDD6) | (1 << DDD5) | (1 << DDD7);

	// jetzt erst default values setzen
	PORTC = (1 << PORTC0) | (1 << PORTC1) | (1 << PORTC2) | (1 << PORTC3);

	// Pin Change Intterupts aufsetzen
	// PCMSK*: Pin Change Masks (welche Pins tragen zum Intterupt bei)
	// PORT C:
	  //PCMSK2 = (1 << PCINT16) | (1 << PCINT17) | (1 << PCINT18) | (1 << PCINT19);
	// Pin Change Interrupt Control Register: PC Interrupt Enable for Port C
	  //PCICR = (1 << PCIE2);

	// INTTERUPTS AUFSETZEN
	EICRA = (1 << ISC20) | (1 << ISC21) // rising FLANKE von INT2 (CA = Load Cycle) erzeugt intterrupt
			| (1 << ISC11) | (1 << ISC10); // rising edge von INT1 (LINE READY) erzeugt intteruupt

	EIMSK = (1 << INT2) | (1 << INT1); // interrupt fuer INT1,2 anschalten


	// Interrupt enable
	sei();
}




int main() {
	start_led();

	Initialize();
	uart_init(38400);
	stdout = &output;
	line_state = LINE_FINISHED; // Startwert
	
	puts("ANELEX Interface Microcontroller  (c) 2010 Sven Koeppel");
	puts("This program is part of the Bull Anelex Project at technikum29");
	puts("Options: 1. Enter text to print. 2. Type ?help for available commands.");
	puts("3. Press Button to print default text. 4. Obey RS232 hardware flow control.");
	stop_led();
	
	for(;;) {
		start_led();
		readline();
		stop_led();

		// process user input
		if(current_line[0] == '?') {
			// if exec_command returns 0, read a new line,
			// else continue processing current_line.
			if(!exec_command())
				continue;
		}

		// if readline() went to fast:
		while(line_state != LINE_FINISHED) {
			puts("Wait for printer finished...");
			_delay_ms(2000);
		}

		print_line(); // returns immediately

		while(line_state == LINE_TRANSMITTING);
		puts("printing - accepting new line");
	}

	return 0; // ende.
}

/*
	byte_t y;
	for(y=0;;y++) {
		// warten, bis BUTTON gedrueckt ist
		if(PINB & (1 << PORTB4)) {
			start_led();
			for(;;) {
				if((PINB & (1 << PORTB4)) == 0)
					break;
			}
			printf("%d: Button gedrueckt\n", counter++);

			for(int zzz=0; zzz < 1; zzz++) {

			// hier koennen wir jetzt mal arbeiten
			// alle strings durchdrucken
			for(int s=0; tests[s][0]; s++) {
				current_string = tests[s];
				printf("%d. Drucke %s\n", s, current_string);

				print_line_feed();
				current_counter = 0;

				for(;;) {
					if(current_counter == LINE_PRINTED) {
						printf("Zeile fertig! :-)\n");
						break;
					}
				}

				stop_led();

			} // for tests

			} // mehrfachtests
		} // for buton
		//printf("%d: Sleeping...\n", counter);
		//counter++;
		//_delay_ms(10);
		// endlosschleife
	}

	return 0;
}
*/
